@@ -1,5 +1,6 @@
 import asyncio
 import datetime
+from enum import Enum
 import logging
 import time
 
@@ -75,7 +76,7 @@ class SmartTub:
 
     def _set_access_token(self, token):
         self.access_token = token
-        self.access_token_data = jwt.decode(self.access_token, verify=False)
+        self.access_token_data = jwt.decode(self.access_token, options={"verify_signature": False})
         self.token_expires_at = self.access_token_data['exp']
 
     async def _refresh_token(self):
@@ -142,11 +143,10 @@ class Account:
 
 
 class Spa:
-    SECONDARY_FILTRATION_MODES = {'FREQUENT', 'INFREQUENT', 'AWAY'}
-    HEAT_MODES = {'ECONOMY', 'DAY', 'AUTO'}
-    LIGHT_MODES = {'PURPLE', 'ORANGE', 'RED', 'YELLOW', 'GREEN', 'AQUA', 'BLUE', 'HIGH_SPEED_WHEEL', 'OFF'}
-    TEMPERATURE_FORMATS = ['FAHRENHEIT', 'CELSIUS']
-    ENERGY_USAGE_INTERVALS = ['DAY', 'MONTH']
+    SecondaryFiltrationMode = Enum('SecondaryFiltrationMode', 'FREQUENT INFREQUENT AWAY')
+    HeatMode = Enum('HeatMode', 'ECONOMY DAY AUTO')
+    TemperatureFormat = Enum('TemperatureFormat', 'FAHRENHEIT CELSIUS')
+    EnergyUsageInterval = Enum('EnergyUsageInterval', 'DAY MONTH')
 
     def __init__(self, api: SmartTub, account: Account, **properties):
         self._api = api
@@ -165,44 +165,42 @@ class Spa:
         return await self.request('GET', 'status')
 
     async def get_pumps(self) -> list:
-        return [SpaPump(self._api, self, **pump_info)
+        return [SpaPump(self, **pump_info)
                 for pump_info in (await self.request('GET', 'pumps'))['pumps']]
 
     async def get_lights(self) -> list:
-        return [SpaLight(self._api, self, **light_info)
+        return [SpaLight(self, **light_info)
                 for light_info in (await self.request('GET', 'lights'))['lights']]
 
     async def get_errors(self) -> list:
-        return (await self.request('GET', 'errors'))['content']
+        return [SpaError(self, **error_info)
+                for error_info in (await self.request('GET', 'errors'))['content']]
 
     async def get_reminders(self) -> dict:
         # API returns both 'reminders' and 'filters', both seem to be identical
-        return [SpaReminder(self._api, self, **reminder_info)
+        return [SpaReminder(self, **reminder_info)
                 for reminder_info in (await self.request('GET', 'reminders'))['reminders']]
 
     async def get_debug_status(self) -> dict:
         return (await self.request('GET', 'debugStatus'))['debugStatus']
 
-    async def get_energy_usage(self, interval: str, start_date: datetime.date, end_date: datetime.date) -> list:
-        assert interval in self.ENERGY_USAGE_INTERVALS
+    async def get_energy_usage(self, interval: EnergyUsageInterval, start_date: datetime.date, end_date: datetime.date) -> list:
         body = {
             "start": start_date.isoformat(),
             "end": end_date.isoformat(),
-            "interval": interval,
+            "interval": interval.name,
         }
         return (await self.request('POST', 'energyUsage', body))['buckets']
 
-    async def set_secondary_filtration_mode(self, mode: str):
-        assert mode in self.SECONDARY_FILTRATION_MODES
+    async def set_secondary_filtration_mode(self, mode: SecondaryFiltrationMode):
         body = {
-            'secondaryFiltrationConfig': mode
+            'secondaryFiltrationConfig': mode.name
         }
         await self.request('PATCH', 'config', body)
 
-    async def set_heat_mode(self, mode: str):
-        assert mode in self.HEAT_MODES
+    async def set_heat_mode(self, mode: HeatMode):
         body = {
-            'heatMode': mode
+            'heatMode': mode.name
         }
         await self.request('PATCH', 'config', body)
 
@@ -216,10 +214,9 @@ class Spa:
     async def toggle_clearray(self, str):
         await self.request('POST', 'clearray/toggle')
 
-    async def set_temperature_format(self, temperature_format: str):
-        assert temperature_format in self.TEMPERATURE_MODES
+    async def set_temperature_format(self, temperature_format: TemperatureFormat):
         body = {
-            'displayTemperatureFormat': temperature_format
+            'displayTemperatureFormat': temperature_format.name
         }
         await self.request('POST', 'config', body)
 
@@ -243,8 +240,7 @@ class Spa:
 
 
 class SpaPump:
-    def __init__(self, api: SmartTub, spa: Spa, **properties):
-        self._api = api
+    def __init__(self, spa: Spa, **properties):
         self.spa = spa
         self.id = properties['id']
         self.speed = properties['speed']
@@ -256,35 +252,41 @@ class SpaPump:
         await self.spa.request('POST', f'pumps/{self.id}/toggle')
 
     def __str__(self):
-        return f'<SpaPump {self.id}>'
+        return f'<SpaPump {self.id}: {self.type}={self.state}>'
 
 
 class SpaLight:
-    def __init__(self, api: SmartTub, spa: Spa, **properties):
-        self._api = api
+    LightMode = Enum('LightMode', 'PURPLE ORANGE RED YELLOW GREEN AQUA BLUE HIGH_SPEED_WHEEL OFF')
+
+    def __init__(self, spa: Spa, **properties):
         self.spa = spa
         self.zone = properties['zone']
-        self.color = properties['color']
+
+        color = properties['color']
+        self.red = color['red']
+        self.green = color['green']
+        self.blue = color['blue']
+        self.white = color['white']
+
         self.intensity = properties['intensity']
         self.mode = properties['mode']
         self.properties = properties
 
-    async def set(self, intensity: int, mode: str):
-        assert mode in self.LIGHT_MODES
-        assert (intensity == 0) == (mode == 'OFF')
+    async def set_mode(self, mode: LightMode, intensity: int):
+        assert (intensity == 0) == (mode == self.LightMode.OFF)
+
         body = {
             'intensity': intensity,
-            'mode': mode,
+            'mode': mode.name,
         }
         await self.spa.request('PATCH', f'lights/{self.zone}', body)
 
     def __str__(self):
-        return f'<SpaLight {self.zone}>'
+        return f'<SpaLight {self.zone}: {self.red}/{self.green}/{self.blue}/{self.white}>'
 
 
 class SpaReminder:
-    def __init__(self, api: SmartTub, spa: Spa, **properties):
-        self._api = api
+    def __init__(self, spa: Spa, **properties):
         self.spa = spa
         self.id = properties['id']
         self.last_updated = dateutil.parser.isoparse(properties['lastUpdated'])
@@ -299,6 +301,21 @@ class SpaReminder:
         return f'<SpaReminder {self.id}>'
 
 
+class SpaError:
+    def __init__(self, spa: Spa, **properties):
+        self.spa = spa
+        self.code = properties['code']
+        self.title = properties['title']
+        self.description = properties['description']
+        self.created_at = dateutil.parser.isoparse(properties['createdAt'])
+        self.updated_at = dateutil.parser.isoparse(properties['updatedAt'])
+        self.active = properties['active']
+        self.error_type = properties['errorType']
+
+    def __str__(self):
+        return f'<SpaError {self.title}>'
+
+
 class LoginFailed(RuntimeError):
     pass
 
